@@ -279,6 +279,122 @@ class DocumentFormatter(Formatter):
             self.print_cyan("  Not executed: ", len(not_executed_examples))
 
 
+class LongFormatter(Formatter):
+    def new_example(self, example):
+        self.print_white(
+            "{}{}: ".format(
+                "*" if example.context.focus else "", example.context.full_name
+            ),
+            end="",
+        )
+
+    def _color_output(self):
+        return sys.stdout.isatty() or self.force_color
+
+    def success(self, example):
+        Formatter.success(self, example)
+        self.print_green(
+            "{focus}{example}{pass_text}".format(
+                focus="*" if example.focus else "",
+                example=example,
+                pass_text="" if self._color_output() else ": PASS",
+            )
+        )
+
+    def fail(self, example, exception):
+        if isinstance(exception, AggregatedExceptions) and 1 == len(
+            exception.exceptions
+        ):
+            exception = exception.exceptions[0]
+
+        Formatter.fail(self, example, exception)
+
+        self.print_red(
+            "{focus}{example}: ".format(
+                focus="*" if example.focus else "", example=example,
+            ),
+            end="",
+        )
+        print(
+            "{ex_class}: {ex_message}".format(
+                ex_class=type(exception).__name__,
+                ex_message=str(exception).split("\n")[0],
+            )
+        )
+
+    def skip(self, example):
+        Formatter.skip(self, example)
+        self.print_yellow(
+            "{focus}{example}{skip_text}".format(
+                focus="*" if example.focus else "",
+                example=example,
+                skip_text="" if self._color_output() else ": SKIP",
+            )
+        )
+
+    def print_failed_example(self, number, example, exception):
+        self.print_white(
+            "  {number}) {context}: {example}".format(
+                number=number, context=example.context.full_name, example=example
+            )
+        )
+        if type(exception) is AggregatedExceptions:
+            exception_list = exception.exceptions
+        else:
+            exception_list = [exception]
+        for number, exception in enumerate(exception_list):
+            self.print_red(
+                "    {number}) {exception_class}: {message}".format(
+                    number=number + 1,
+                    exception_class=exception.__class__.__name__,
+                    message="\n    ".join(str(exception).split("\n")),
+                )
+            )
+            for path, line, function_name, text in traceback.extract_tb(
+                exception.__traceback__
+            ):
+                if not self.show_testslide_stack_trace and path.startswith(
+                    os.path.dirname(__file__)
+                ):
+                    continue
+                if self.trim_stack_trace_path_prefix:
+                    split = path.split(self.trim_stack_trace_path_prefix)
+                    if len(split) == 2 and not split[0]:
+                        path = split[1]
+                self.print_cyan(
+                    '      File "{}", line {}, in {}\n'
+                    "        {}".format(path, line, function_name, text)
+                )
+
+    def finish(self, not_executed_examples):
+        Formatter.finish(self, not_executed_examples)
+        success = len(self.results["success"])
+        fail = len(self.results["fail"])
+        skip = len(self.results["skip"])
+        total = success + fail + skip
+        if self.results["fail"]:
+            self.print_red("\nFailures:")
+            for number, result in enumerate(self.results["fail"]):
+                print("")
+                self.print_failed_example(
+                    number + 1, result["example"], result["exception"]
+                )
+        print("")
+        self.print_white(
+            "Finished %s example(s) in %.1fs: ." % (total, self.duration_secs)
+        )
+        if self.import_secs > 2:
+            self.print_white("Imports took: %.1fs" % (self.import_secs))
+        if success:
+            self.print_green("  Successful: ", success)
+        if fail:
+            self.print_red("  Failed: ", fail)
+        if skip:
+            self.print_yellow("  Skipped: ", skip)
+        if not_executed_examples:
+            self.print_cyan("  Not executed: ", len(not_executed_examples))
+
+
 class Runner(object):
     """
     Execute examples contained in given contexts.
