@@ -2,55 +2,14 @@
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
-
 import dis
 import copy
-import functools
 import inspect
 import os.path
+import testslide.lib
+from typing import Optional, Any
 
-from unittest.mock import _must_skip
-from .lib import _validate_function_signature
-
-
-def _wrap_signature_and_type_validation(value, template, attr_name):
-    if isinstance(template, StrictMock):
-        if "_template" in template.__dict__:
-            template = template._template
-        else:
-            return value
-
-    # This covers runtime attributes
-    if not hasattr(template, attr_name):
-        return value
-
-    callable_template = getattr(template, attr_name)
-
-    # FIXME decouple from _must_skip. It tells when self should be skipped
-    # for signature validation.
-    if _must_skip(template, attr_name, isinstance(template, type)):
-        callable_template = functools.partial(callable_template, None)
-
-    try:
-        signature = inspect.signature(callable_template, follow_wrapped=False)
-    except ValueError:
-        signature = None
-
-    def with_sig_check(*args, **kwargs):
-        if signature:
-            try:
-                signature.bind(*args, **kwargs)
-            except TypeError as e:
-                raise TypeError(
-                    "{}, {}: {}".format(repr(template), repr(attr_name), str(e))
-                )
-            argspec = inspect.getfullargspec(callable_template)
-            validation_errors = _validate_function_signature(argspec, args, kwargs)
-            if validation_errors:
-                raise TypeError(validation_errors)
-        return value(*args, **kwargs)
-
-    return with_sig_check
+import testslide.mock_callable
 
 
 class UndefinedAttribute(BaseException):
@@ -393,6 +352,7 @@ class StrictMock(object):
         name=None,
         default_context_manager=False,
         signature_validation=True,
+        type_validation=True,
     ):
         """
         For every new instance of StrictMock we dynamically create a subclass of
@@ -458,8 +418,28 @@ class StrictMock(object):
                 self.__aenter__ = aenter
                 self.__aexit__ = aexit
 
+    def _get_caller_frame(self, depth):
+        # Adding extra 3 to account for the stack:
+        #   _get_caller_frame
+        #   _get_caller
+        #   __init__
+        depth = depth + 3
+        current_frame = inspect.currentframe()
+        while current_frame:
+            depth -= 1
+            if not depth:
+                break
+
+            current_frame = current_frame.f_back
+
+        return current_frame
+
     def _get_caller(self, depth):
-        frameinfo = inspect.getframeinfo(inspect.stack()[depth + 1][0])
+        # Doing inspect.stack will retrieve the whole stack, including context
+        # and that is really slow, this only retrieves the minimum, and does
+        # not read the file contents.
+        caller_frame = self._get_caller_frame(depth)
+        frameinfo = inspect.getframeinfo(caller_frame, context=0)
         filename = frameinfo.filename
         lineno = frameinfo.lineno
         if self.TRIM_PATH_PREFIX:
@@ -478,6 +458,7 @@ class StrictMock(object):
         name=None,
         default_context_manager=False,
         signature_validation=True,
+        type_validation=True,
     ):
         """
         template: Template class to be used as a template for the mock.
@@ -495,7 +476,14 @@ class StrictMock(object):
         the attribute will change. Setting this value to False disables
         signature validation, and should only be used when type() is required
         to not change.
+        type_validation: use type hinting information from template to validate
+        that mock attribute types match them. Type validation also happens for
+        callable attributes (instance/static/class methods) calls.
         """
+        if not signature_validation and type_validation:
+            raise ValueError(
+                "Type Validation is only available with Signature validation turned on"
+            )
         if template and not inspect.isclass(template):
             raise ValueError("Template must be a class.")
         self.__dict__["_template"] = template
@@ -503,6 +491,7 @@ class StrictMock(object):
         self.__dict__["_runtime_attrs"] = runtime_attrs or []
         self.__dict__["_name"] = name
         self.__dict__["_signature_validation"] = signature_validation
+        self.__dict__["_type_validation"] = type_validation
         self.__dict__["__caller"] = self._get_caller(1)
 
         self._setup_magic_methods()
@@ -567,6 +556,87 @@ class StrictMock(object):
     def _is_magic_method(name):
         return name.startswith("__") and name.endswith("__")
 
+    def _validate_attribute_type(self, name, value):
+        if not self.__dict__["_type_validation"]:
+            return
+
+        if hasattr(self._template, "__annotations__"):
+            annotations = self._template.__annotations__
+            if name in annotations:
+                testslide.lib._validate_argument_type(annotations[name], name, value)
+
+    def _validate_and_wrap_mock_value(self, name, value):
+        if self._template:
+            if not self._template_has_attr(name):
+                raise NonExistentAttribute(self, name)
+
+            self._validate_attribute_type(name, value)
+
+            if hasattr(self._template, name):
+                template_value = getattr(self._template, name)
+                if callable(template_value):
+                    if not callable(value):
+                        raise NonCallableValue(self, name)
+                    if self.__dict__["_signature_validation"]:
+                        signature_validation_wrapper = testslide.lib._wrap_signature_validation(
+                            value,
+                            self._template,
+                            name,
+                            self.__dict__["_type_validation"],
+                        )
+
+                        if inspect.iscoroutinefunction(template_value):
+
+                            async def awaitable_return_validation_wrapper(
+                                *args, **kwargs
+                            ):
+                                result_awaitable = signature_validation_wrapper(
+                                    *args, **kwargs
+                                )
+                                if not inspect.isawaitable(result_awaitable):
+                                    raise NonAwaitableReturn(self, name)
+
+                                return_value = await result_awaitable
+                                if self.__dict__["_type_validation"] and not isinstance(
+                                    value, testslide.mock_callable._CallableMock
+                                ):
+                                    testslide.lib._validate_return_type(
+                                        template_value, return_value
+                                    )
+                                return return_value
+
+                            callable_value = awaitable_return_validation_wrapper
+                        else:
+
+                            def return_validation_wrapper(*args, **kwargs):
+                                return_value = signature_validation_wrapper(
+                                    *args, **kwargs
+                                )
+                                if self.__dict__["_type_validation"] and not isinstance(
+                                    value, testslide.mock_callable._CallableMock
+                                ):
+                                    testslide.lib._validate_return_type(
+                                        template_value, return_value
+                                    )
+                                return return_value
+
+                            callable_value = return_validation_wrapper
+                    else:
+                        callable_value = None
+                    return _MethodProxy(value=value, callable_value=callable_value)
+            else:
+                if callable(value):
+                    # We don't really need the proxy here, but it serves the
+                    # double purpose of swallowing self / cls when needed.
+                    return _MethodProxy(value=value)
+        else:
+            if callable(value):
+                # We don't really need the proxy here, but it serves the
+                # double purpose of swallowing self / cls when needed.
+                return _MethodProxy(value=value)
+
+        return value
+
     def __setattr__(self, name, value):
         if self._is_magic_method(name):
             # ...check whether we're allowed to mock...
@@ -580,52 +650,7 @@ class StrictMock(object):
                     f"StrictMock does not implement support for {name}"
                 )
 
-        mock_value = value
-        if self._template:
-            if not self._template_has_attr(name):
-                raise NonExistentAttribute(self, name)
-
-            if hasattr(self._template, name):
-                template_value = getattr(self._template, name)
-                if callable(template_value):
-                    if not callable(value):
-                        raise NonCallableValue(self, name)
-
-                    if self.__dict__["_signature_validation"]:
-                        signature_validation_wrapper = _wrap_signature_and_type_validation(
-                            value, self._template, name
-                        )
-                        if inspect.iscoroutinefunction(template_value):
-
-                            async def awaitable_return_validation_wrapper(
-                                *args, **kwargs
-                            ):
-                                return_value = signature_validation_wrapper(
-                                    *args, **kwargs
-                                )
-                                if not inspect.isawaitable(return_value):
-                                    raise NonAwaitableReturn(self, name)
-                                return await return_value
-
-                            callable_value = awaitable_return_validation_wrapper
-                        else:
-                            callable_value = signature_validation_wrapper
-                    else:
-                        callable_value = None
-                    mock_value = _MethodProxy(
-                        value=value, callable_value=callable_value
-                    )
-            else:
-                if callable(value):
-                    # We don't really need the proxy here, but it serves the
-                    # double purpose of swallowing self / cls when needed.
-                    mock_value = _MethodProxy(value=value)
-        else:
-            if callable(value):
-                # We don't really need the proxy here, but it serves the
-                # double purpose of swallowing self / cls when needed.
-                mock_value = _MethodProxy(value=value)
-
+        mock_value = self._validate_and_wrap_mock_value(name, value)
         setattr(type(self), name, mock_value)
 
     def __getattr__(self, name):
@@ -668,6 +693,7 @@ class StrictMock(object):
             runtime_attrs=self._runtime_attrs,
             name=self._name,
             signature_validation=self._signature_validation,
+            type_validation=self._type_validation,
         )
         self_copy.__dict__["__caller"] = self._get_caller(2)
         return self_copy
@@ -703,3 +729,13 @@ class StrictMock(object):
             value = copy.deepcopy(type(self).__dict__[name], memo)
             setattr(type(self_copy), name, value)
         return self_copy
+
+
+def _extract_StrictMock_template(mock_obj) -> Optional[Any]:
+    if "_template" in mock_obj.__dict__ and mock_obj._template is not None:
+        return mock_obj._template
+
+    return None
+
+
+testslide.lib.MOCK_TEMPLATE_EXTRACTORS[StrictMock] = _extract_StrictMock_template
